@@ -36,6 +36,7 @@ from .utils import (
     ByteBuffer,
     _CamelCaseDataObject,
 )
+from .features import webauthn_json_mapping
 from enum import Enum, EnumMeta, unique, IntFlag
 from dataclasses import dataclass, field
 from typing import Any, Mapping, Optional, Sequence, Tuple, Union, cast
@@ -113,7 +114,7 @@ class AttestedCredentialData(bytes):
     @classmethod
     def create(
         cls, aaguid: bytes, credential_id: bytes, public_key: CoseKey
-    ) -> "AttestedCredentialData":
+    ) -> AttestedCredentialData:
         """Create an AttestedCredentialData by providing its components.
 
         :param aaguid: The AAGUID of the authenticator.
@@ -129,7 +130,7 @@ class AttestedCredentialData(bytes):
         )
 
     @classmethod
-    def unpack_from(cls, data: bytes) -> Tuple["AttestedCredentialData", bytes]:
+    def unpack_from(cls, data: bytes) -> Tuple[AttestedCredentialData, bytes]:
         """Unpack an AttestedCredentialData from a byte string, returning it and
         any remaining data.
 
@@ -141,9 +142,7 @@ class AttestedCredentialData(bytes):
         return cls.create(aaguid, cred_id, pub_key), rest
 
     @classmethod
-    def from_ctap1(
-        cls, key_handle: bytes, public_key: bytes
-    ) -> "AttestedCredentialData":
+    def from_ctap1(cls, key_handle: bytes, public_key: bytes) -> AttestedCredentialData:
         """Create an AttestatedCredentialData from a CTAP1 RegistrationData instance.
 
         :param key_handle: The CTAP1 credential key_handle.
@@ -169,15 +168,25 @@ class AuthenticatorData(bytes):
     :ivar extensions: Authenticator extensions, if available.
     """
 
-    @unique
     class FLAG(IntFlag):
         """Authenticator data flags
 
         See https://www.w3.org/TR/webauthn/#sec-authenticator-data for details
         """
 
+        # Names used in WebAuthn
+        UP = 0x01
+        UV = 0x04
+        BE = 0x08
+        BS = 0x10
+        AT = 0x40
+        ED = 0x80
+
+        # Aliases (for historical purposes)
         USER_PRESENT = 0x01
         USER_VERIFIED = 0x04
+        BACKUP_ELIGIBILITY = 0x08
+        BACKUP_STATE = 0x10
         ATTESTED = 0x40
         EXTENSION_DATA = 0x80
 
@@ -196,13 +205,13 @@ class AuthenticatorData(bytes):
         object.__setattr__(self, "counter", reader.unpack(">I"))
         rest = reader.read()
 
-        if self.flags & AuthenticatorData.FLAG.ATTESTED:
+        if self.flags & AuthenticatorData.FLAG.AT:
             credential_data, rest = AttestedCredentialData.unpack_from(rest)
         else:
             credential_data = None
         object.__setattr__(self, "credential_data", credential_data)
 
-        if self.flags & AuthenticatorData.FLAG.EXTENSION_DATA:
+        if self.flags & AuthenticatorData.FLAG.ED:
             extensions, rest = cbor.decode_from(rest)
         else:
             extensions = None
@@ -241,36 +250,28 @@ class AuthenticatorData(bytes):
         )
 
     def is_user_present(self) -> bool:
-        """Return true if the User Present flag is set.
-
-        :return: True if User Present is set, False otherwise.
-        :rtype: bool
-        """
-        return bool(self.flags & AuthenticatorData.FLAG.USER_PRESENT)
+        """Return true if the User Present flag is set."""
+        return bool(self.flags & AuthenticatorData.FLAG.UP)
 
     def is_user_verified(self) -> bool:
-        """Return true if the User Verified flag is set.
+        """Return true if the User Verified flag is set."""
+        return bool(self.flags & AuthenticatorData.FLAG.UV)
 
-        :return: True if User Verified is set, False otherwise.
-        :rtype: bool
-        """
-        return bool(self.flags & AuthenticatorData.FLAG.USER_VERIFIED)
+    def is_backup_eligible(self) -> bool:
+        """Return true if the Backup Eligibility flag is set."""
+        return bool(self.flags & AuthenticatorData.FLAG.BE)
+
+    def is_backed_up(self) -> bool:
+        """Return true if the Backup State flag is set."""
+        return bool(self.flags & AuthenticatorData.FLAG.BS)
 
     def is_attested(self) -> bool:
-        """Return true if the Attested credential data flag is set.
-
-        :return: True if Attested credential data is set, False otherwise.
-        :rtype: bool
-        """
-        return bool(self.flags & AuthenticatorData.FLAG.ATTESTED)
+        """Return true if the Attested credential data flag is set."""
+        return bool(self.flags & AuthenticatorData.FLAG.AT)
 
     def has_extension_data(self) -> bool:
-        """Return true if the Extenstion data flag is set.
-
-        :return: True if Extenstion data is set, False otherwise.
-        :rtype: bool
-        """
-        return bool(self.flags & AuthenticatorData.FLAG.EXTENSION_DATA)
+        """Return true if the Extenstion data flag is set."""
+        return bool(self.flags & AuthenticatorData.FLAG.ED)
 
 
 @dataclass(init=False, frozen=True)
@@ -321,7 +322,7 @@ class AttestationObject(bytes):  # , Mapping[str, Any]):
             "fido-u2f",
             AuthenticatorData.create(
                 app_param,
-                AuthenticatorData.FLAG.ATTESTED | AuthenticatorData.FLAG.USER_PRESENT,
+                AuthenticatorData.FLAG.AT | AuthenticatorData.FLAG.UP,
                 0,
                 AttestedCredentialData.from_ctap1(
                     registration.key_handle, registration.public_key
@@ -371,7 +372,7 @@ class CollectedClientData(bytes):
                     "type": type,
                     "challenge": encoded_challenge,
                     "origin": origin,
-                    "cross_origin": cross_origin,
+                    "crossOrigin": cross_origin,
                     **kwargs,
                 },
                 separators=(",", ":"),
@@ -408,6 +409,12 @@ class _StringEnum(str, Enum, metaclass=_StringEnumMeta):
     """
 
 
+_b64_metadata = dict(
+    serialize=lambda x: websafe_encode(x) if webauthn_json_mapping.enabled else x,
+    deserialize=lambda x: websafe_decode(x) if webauthn_json_mapping.enabled else x,
+)
+
+
 @unique
 class AttestationConveyancePreference(_StringEnum):
     NONE = "none"
@@ -441,6 +448,7 @@ class AuthenticatorTransport(_StringEnum):
     USB = "usb"
     NFC = "nfc"
     BLE = "ble"
+    HYBRID = "hybrid"
     INTERNAL = "internal"
 
 
@@ -463,7 +471,7 @@ class PublicKeyCredentialRpEntity(_CamelCaseDataObject):
 @dataclass(eq=False, frozen=True)
 class PublicKeyCredentialUserEntity(_CamelCaseDataObject):
     name: str
-    id: bytes
+    id: bytes = field(metadata=_b64_metadata)
     display_name: Optional[str] = None
 
 
@@ -483,7 +491,7 @@ class PublicKeyCredentialParameters(_CamelCaseDataObject):
 @dataclass(eq=False, frozen=True)
 class PublicKeyCredentialDescriptor(_CamelCaseDataObject):
     type: PublicKeyCredentialType
-    id: bytes
+    id: bytes = field(metadata=_b64_metadata)
     transports: Optional[Sequence[AuthenticatorTransport]] = None
 
     @classmethod
@@ -523,7 +531,7 @@ class AuthenticatorSelectionCriteria(_CamelCaseDataObject):
 class PublicKeyCredentialCreationOptions(_CamelCaseDataObject):
     rp: PublicKeyCredentialRpEntity
     user: PublicKeyCredentialUserEntity
-    challenge: bytes
+    challenge: bytes = field(metadata=_b64_metadata)
     pub_key_cred_params: Sequence[PublicKeyCredentialParameters] = field(
         metadata=dict(deserialize=PublicKeyCredentialParameters._deserialize_list),
     )
@@ -539,12 +547,12 @@ class PublicKeyCredentialCreationOptions(_CamelCaseDataObject):
 
 @dataclass(eq=False, frozen=True)
 class PublicKeyCredentialRequestOptions(_CamelCaseDataObject):
-    challenge: bytes
+    challenge: bytes = field(metadata=_b64_metadata)
     timeout: Optional[int] = None
     rp_id: Optional[str] = None
     allow_credentials: Optional[Sequence[PublicKeyCredentialDescriptor]] = field(
         default=None,
-        metadata={"deserialize": PublicKeyCredentialDescriptor._deserialize_list},
+        metadata=dict(deserialize=PublicKeyCredentialDescriptor._deserialize_list),
     )
     user_verification: Optional[UserVerificationRequirement] = None
     extensions: Optional[Mapping[str, Any]] = None
@@ -552,16 +560,89 @@ class PublicKeyCredentialRequestOptions(_CamelCaseDataObject):
 
 @dataclass(eq=False, frozen=True)
 class AuthenticatorAttestationResponse(_CamelCaseDataObject):
-    client_data: bytes
-    attestation_object: AttestationObject
+    client_data: CollectedClientData = field(
+        metadata=dict(
+            _b64_metadata,
+            name="clientDataJSON",
+        )
+    )
+    attestation_object: AttestationObject = field(metadata=_b64_metadata)
     extension_results: Optional[Mapping[str, Any]] = None
 
+    def __getitem__(self, key):
+        if key == "clientData" and not webauthn_json_mapping.enabled:
+            return self.client_data
+        return super().__getitem__(key)
+
+    @classmethod
+    def from_dict(cls, data: Optional[Mapping[str, Any]]):
+        if data is not None and not webauthn_json_mapping.enabled:
+            value = dict(data)
+            value["clientDataJSON"] = value.pop("clientData", None)
+            data = value
+        return super().from_dict(data)
+
 
 @dataclass(eq=False, frozen=True)
 class AuthenticatorAssertionResponse(_CamelCaseDataObject):
-    client_data: bytes
-    authenticator_data: AuthenticatorData
-    signature: bytes
-    user_handle: bytes
-    credential_id: bytes
+    client_data: CollectedClientData = field(
+        metadata=dict(
+            _b64_metadata,
+            name="clientDataJSON",
+        )
+    )
+    authenticator_data: AuthenticatorData = field(metadata=_b64_metadata)
+    signature: bytes = field(metadata=_b64_metadata)
+    user_handle: Optional[bytes] = field(metadata=_b64_metadata, default=None)
+    credential_id: Optional[bytes] = field(metadata=_b64_metadata, default=None)
     extension_results: Optional[Mapping[str, Any]] = None
+
+    def __getitem__(self, key):
+        if key == "clientData" and not webauthn_json_mapping.enabled:
+            return self.client_data
+        return super().__getitem__(key)
+
+    @classmethod
+    def from_dict(cls, data: Optional[Mapping[str, Any]]):
+        if data is not None and not webauthn_json_mapping.enabled:
+            value = dict(data)
+            value["clientDataJSON"] = value.pop("clientData", None)
+            data = value
+        print("parse assertion from", data)
+        return super().from_dict(data)
+
+
+@dataclass(eq=False, frozen=True)
+class RegistrationResponse(_CamelCaseDataObject):
+    id: bytes = field(metadata=_b64_metadata)
+    response: AuthenticatorAttestationResponse
+    authenticator_attachment: Optional[AuthenticatorAttachment] = None
+    client_extension_results: Optional[Mapping] = None
+    type: Optional[PublicKeyCredentialType] = None
+
+    def __post_init__(self):
+        webauthn_json_mapping.require()
+        super().__post_init__()
+
+
+@dataclass(eq=False, frozen=True)
+class AuthenticationResponse(_CamelCaseDataObject):
+    id: bytes = field(metadata=_b64_metadata)
+    response: AuthenticatorAssertionResponse
+    authenticator_attachment: Optional[AuthenticatorAttachment] = None
+    client_extension_results: Optional[Mapping] = None
+    type: Optional[PublicKeyCredentialType] = None
+
+    def __post_init__(self):
+        webauthn_json_mapping.require()
+        super().__post_init__()
+
+
+@dataclass(eq=False, frozen=True)
+class CredentialCreationOptions(_CamelCaseDataObject):
+    public_key: PublicKeyCredentialCreationOptions
+
+
+@dataclass(eq=False, frozen=True)
+class CredentialRequestOptions(_CamelCaseDataObject):
+    public_key: PublicKeyCredentialRequestOptions
